<?php

/**
 * @file
 * Logic for warden output
 */

/**
 * Return JSON formatted module information.
 */
function warden_status_page() {
  global $base_url;

  drupal_page_is_cacheable(FALSE);

  $system_modules = system_list('module_enabled');
  $system_themes = list_themes(TRUE);

  $res = array(
    "core" => array(),
    "contrib" => array(),
    "custom" => array(),
  );

  // Go over themes.
  if (variable_get('warden_match_contrib', TRUE)) {
    foreach ($system_themes as $theme) {
      if (isset($theme->info['package']) && $theme->info['package'] == 'Core') {
        continue;
      }

      if (isset($theme->info['version'])) {
        $res['contrib'][$theme->name] = array("version" => $theme->info['version']);
      }
    }
  }

  // Go over modules.
  foreach ($system_modules as $module => $module_info) {
    $filename = $module_info->filename;

    // Match for custom modules.
    if (variable_get('warden_match_custom', TRUE)) {
      $regex = variable_get('warden_preg_match_custom', '{^sites\/([A-z,\.,\-]+)\/modules\/custom\/*}');
      if (preg_match($regex, $filename)) {
        // if this is part of a project, only set the project.
        if (isset($module_info->info['project'])) {
          $res['custom'][$module_info->info['project']] = array("version" => $module_info->info['version']);
        }
        else {
          $res['custom'][$module] = array("version" => $module_info->info['version']);
        }
      }
    }
    else {
      $res['custom'] = "disabled";
    }

    // Match for contrib modules.
    if (variable_get('warden_match_contrib', TRUE)) {
      $regex = variable_get('warden_preg_match_contrib', '{^sites\/([A-z,\.,\-]+)\/modules\/contrib\/*}');

      if (preg_match($regex, $filename)) {
        // if this is part of a project, only set the project.
        if (isset($module_info->info['project'])) {
          $res['contrib'][$module_info->info['project']] = array("version" => $module_info->info['version']);
        }
        else {
          $res['contrib'][$module] = array("version" => $module_info->info['version']);
        }
      }
    }
    else {
      $res['contrib'] = "disabled";
    }

    // Core modules.
    if (variable_get('warden_match_core', TRUE)) {
      if (strtolower($module_info->info['package']) == "core") {
        $res['core'][$module] = array("version" => $module_info->info['version']);
      }
    }
    else {
      $res['core'] = "disabled";
    }
  }

  // Include Drupal core version.
  if (!isset($res['core']['drupal'])) {
    $res['core']['drupal'] = array("version" => VERSION);
  }

  $res['site_name'] = variable_get('site_name', $base_url);

  try {
    $encrypted_message = warden_get_encryption()->encrypt($res);

    drupal_json_output(array(
      'data' => $encrypted_message,
    ));
  } catch (Exception $e) {
    watchdog_exception('warden', $e);
    drupal_json_output(array('error' => 'Internal fault'));
  }
}
