<?php

/**
 * @file
 * Encryption logic for warden
 */

class WardenEncryption {

  /**
   * @var string
   */
  protected $wardenUrl;

  /**
   * @var string
   */
  protected $wardenPublicKey = '';

  /**
   * @var string
   */
  protected $username = '';

  /**
   * @var string
   */
  protected $password = '';

  /**
   * @param string $warden_url
   *   The URL to the server.
   * @param string $username
   *   (optional) The basic HTTP username of warden if set.
   * @param string $password
   * (optional) The basic HTTP password of warden if set.
   */
  public function __construct($warden_url, $username = '', $password = '') {
    $this->wardenUrl = $warden_url;
    $this->username = $username;
    $this->password = $password;
  }

  /**
   * Get the public key.
   */
  public function getPublicKey() {

    if (empty($this->wardenPublicKey)) {
      $public_key_url = $this->wardenUrl . '/public-key';
      $options = array();

      if (!empty($this->username)) {
        $options['headers']['Authorization'] = 'Basic ' . base64_encode($this->username . ':' . $this->password);
      }

      $result = drupal_http_request($public_key_url, $options);

      if ($result->code != 200) {
        watchdog('warden', '@url : @code', array('@url' => $public_key_url, '@code' => $result->code), WATCHDOG_ERROR);
        throw new Exception('Unable to retrieve public key from Warden');
      }

      $this->wardenPublicKey = base64_decode($result->data);
    }

    return $this->wardenPublicKey;
  }

  /**
   * Encrypt a plaintext message.
   *
   * @param mixed $data
   *   The data to encrypt for transport.
   *
   * @return string
   *   The encoded message
   *
   * @throws
   *   Exception if there is a problem
   */
  public function encrypt($data) {
    $plaintext = json_encode($data);

    $public_key = $this->getPublicKey();

    $result = openssl_seal($plaintext, $message, $keys, array($public_key));

    if ($result === FALSE || empty($keys[0]) || empty($message) || $message === $plaintext) {
      throw new Exception('Unable to encrypt a message: ' . openssl_error_string());
    }

    $envelope = (object) array(
      'key' => base64_encode($keys[0]),
      'message' => base64_encode($message),
    );

    return base64_encode(json_encode($envelope));
  }

}
